@@ -6,12 +6,11 @@ This package contains all CLI command implementations.
 
 from specfact_cli.commands import (
     analyze,
-    bridge,
+    auth,
     contract_cmd,
     drift,
     enforce,
     generate,
-    implement,
     import_cmd,
     init,
     migrate,
@@ -21,17 +20,17 @@ from specfact_cli.commands import (
     sdd,
     spec,
     sync,
+    validate,
 )
 
 
 __all__ = [
     "analyze",
-    "bridge",
+    "auth",
     "contract_cmd",
     "drift",
     "enforce",
     "generate",
-    "implement",
     "import_cmd",
     "init",
     "migrate",
@@ -42,4 +41,5 @@ __all__ = [
     "sdd",
     "spec",
     "sync",
+    "validate",
 ]
